#include "rppgui.h"

#include "rppselection.h"
#include "rppdirectories.h"
#include "rppdbhandling.h"
#include "rppname.h"
#include "rppdocumentationdirs.h"
#include "rppoptions.h"
#include "rppoutput.h"
#include "codeinformationrepository.h"
#include "cppsupportpart.h"

#include <kwizard.h>
#include <kdirselectdialog.h>
#include <kpushbutton.h>
#include <klistview.h>
#include <kstandarddirs.h>
#include <kmessagebox.h>
#include <klineedit.h>

#include <qlistview.h>
#include <qstring.h>
#include <qwidget.h>
#include <qobject.h>
#include <qpushbutton.h>
#include <qradiobutton.h>
#include <qcheckbox.h>
#include <qstringlist.h>
#include <qfile.h>
#include <qdir.h>
#include <qptrlist.h>

RPPGui::RPPGui(CodeInformationRepository* rep, CppSupportPart* sup): m_repository(rep), m_cpppart(sup) {
	m_wizard = new KWizard(0, "r++ wizard", true, WDestructiveClose);

	m_selection = new RPPSelectionWidget(m_wizard, "r++ selection", false);
	m_wizard->addPage(m_selection,"What do you want to do?");
	m_wizard->setHelpEnabled(m_selection, false);

	m_dbname = new RPPNameWidget(m_wizard, "r++ db name", false);
	m_wizard->addPage(m_dbname, "database name");
	m_wizard->setHelpEnabled(m_dbname, false);

	m_directories = new RPPDirWidget(m_wizard, "r++ dirs", false);
	m_wizard->addPage(m_directories,"add parsing directories");
	m_wizard->setHelpEnabled(m_directories, false);

	m_options = new RPPOptionsWidget(m_wizard, "r++ options");
	m_wizard->addPage(m_options, "options");
	m_wizard->setHelpEnabled(m_options, false);

	m_docudirectories = new RPPDocumentationWidget(m_wizard, "r++ documentation", false);
	m_wizard->addPage(m_docudirectories, "add documentation directories");
	m_wizard->setHelpEnabled(m_docudirectories, false);

	m_dbmanager = new RPPDbManagerWidget(m_wizard, "r++ manager", false);
	m_wizard->addPage(m_dbmanager, "database management");
	m_wizard->setHelpEnabled(m_dbmanager, false);

	m_wizard->setAppropriate(m_dbname, false);
	m_wizard->setAppropriate(m_docudirectories, false);
	m_wizard->setAppropriate(m_directories, false);
	m_wizard->setAppropriate(m_options, false);
	m_wizard->setAppropriate(m_dbmanager, true);


	m_wizard->setNextEnabled(m_docudirectories, false);
	m_wizard->setNextEnabled(m_dbmanager, false);
	m_wizard->setFinishEnabled(m_dbmanager, true);
	m_wizard->setFinishEnabled(m_docudirectories, true);

	connect(m_selection->m_optmanage, SIGNAL(toggled(bool)), this, SLOT(slotNext(bool)));
	connect(m_wizard->finishButton(), SIGNAL(clicked()), this, SLOT(slotFinish()));
	connect(m_directories->m_btnadd, SIGNAL(clicked()), this, SLOT(slotAddDirectories()));
	connect(m_docudirectories->m_btnadd, SIGNAL(clicked()), this, SLOT(slotAddDirectories()));
	connect(m_directories->m_btnremove, SIGNAL(clicked()), this, SLOT(slotRemoveDirectories()));
	connect(m_docudirectories->m_btnremove, SIGNAL(clicked()), this, SLOT(slotRemoveDirectories()));
	connect(m_dbmanager->m_btndelete, SIGNAL(clicked()), this, SLOT(slotDeleteDb()));
	connect(m_dbmanager->m_btnenable, SIGNAL(clicked()), this, SLOT(slotEnableDb()));
	connect(m_dbmanager->m_btndisable, SIGNAL(clicked()), this, SLOT(slotDisableDb()));
	connect(m_wizard, SIGNAL(selected(const QString&)), this, SLOT(slotManager(const QString&)));
	connect(m_wizard, SIGNAL(destroyed()), this, SLOT(slotClose()));
	KStandardDirs stddir;
	m_datadir = stddir.localkdedir() + "/" + KStandardDirs::kde_default( "data" ) +  "/kdevcppsupport/pcs/";
	if( !QFile::exists(m_datadir) ){
		if (KMessageBox::warningYesNo(m_wizard, "*error* " +m_datadir+ " doesn't exists!\nThe directory is needed for the function of this tool.\nDo You want me to create the directory?", "Error!") == KMessageBox::Yes){
			QDir newdir = QDir();
			if (!newdir.mkdir (m_datadir))
				KMessageBox::sorry(m_wizard, "Sorry, i was unable to create the desired directory, please check permissions!");
				return;
		} else{
			return;
		}
	}
 };

RPPGui::~RPPGui(){
	if (m_wizard)
		delete m_wizard;
	delete m_selection;
	delete m_directories;
	delete m_docudirectories;
	delete m_dbmanager;
	delete m_dbname;
	delete m_options;
};

void RPPGui::show(){
	m_wizard->show();
};

void RPPGui::slotClose(){
	delete this;
}

void RPPGui::slotNext(bool ismanager){
		if (ismanager){
			m_wizard->setAppropriate(m_dbname, false);
			m_wizard->setAppropriate(m_directories, false);
			m_wizard->setAppropriate(m_dbmanager, true);
			m_wizard->setAppropriate(m_docudirectories, false);
			m_wizard->setAppropriate(m_options, false);
		}
		else{
			m_wizard->setAppropriate(m_dbmanager, false);
			m_wizard->setAppropriate(m_dbname, true);
			m_wizard->setAppropriate(m_directories, true);
			m_wizard->setAppropriate(m_docudirectories, true);
			m_wizard->setAppropriate(m_options, true);
		}
};

void RPPGui::slotAddDirectories(){
 	QString dir = KDirSelectDialog::selectDirectory().path();
	if (m_wizard->currentPage() == m_directories){
		m_directories->m_lbodir->insertItem(dir);
	} else{
		m_docudirectories->m_lbodir->insertItem(dir);
	}
};

void RPPGui::slotRemoveDirectories(){
	if (m_wizard->currentPage() == m_directories){
		for(int i = 0; i < m_directories->m_lbodir->count(); ++i)
			if (m_directories->m_lbodir->isSelected(i))
				m_directories->m_lbodir->removeItem(i--);

	} else{
		for(int i = 0; i < m_docudirectories->m_lbodir->count(); ++i)
			if (m_docudirectories->m_lbodir->isSelected(i))
				m_docudirectories->m_lbodir->removeItem(i--);
	}
};

void RPPGui::slotDeleteDb(){
    KStandardDirs stddir;
	QDir dir (m_datadir );
	QListViewItem* i = m_dbmanager->m_lstdb->firstChild();
	while (i){
		if (m_dbmanager->m_lstdb->isSelected(i)){
			if (KMessageBox::questionYesNo(m_wizard, "Are you sure you want to delete " + (*i).text(0)+"?") == KMessageBox::Yes){
				dir.setNameFilter((*i).text(0)+".*");
				QStringList files = dir.entryList();
				bool isremoved = true;
				for (size_t j = 0; j < files.count(); ++j){
					isremoved = QFile::remove(dir.path()+"/"+*(files.at(j)));
				}
				if (isremoved){
					QListViewItem* j = i;
					i = i->nextSibling();
					delete j;
					continue;
				} else
					KMessageBox::error(m_wizard, "unable to delete database");
			}
		}
		i = i->nextSibling();
	}
};

void RPPGui::slotEnableDb(){
	QDir dir(m_datadir);
	dir.setNameFilter("*.db");
	QStringList files = dir.entryList();

	QListViewItem* it = m_dbmanager->m_lstdb->firstChild();
	while (it){
		if (m_dbmanager->m_lstdb->isSelected(it))
			for (unsigned int i = 0; i < files.count(); ++i)
				if ((*(files.at(i))).left((*files.at(i)).length() - 3) == (*it).text(0)){

					m_repository->addCatalog(QString::number(i+1), m_cpppart->catalogList().at(i));
					it->setText(1,"yes");
					break;
				}

		it = it->nextSibling();
	}
};

void RPPGui::slotDisableDb(){
QDir dir(m_datadir);
	dir.setNameFilter("*.db");
	QStringList files = dir.entryList();

	QListViewItem* it = m_dbmanager->m_lstdb->firstChild();
	while (it){
		if (m_dbmanager->m_lstdb->isSelected(it))
			for (unsigned int i = 0; i < files.count(); ++i)
				if ((*(files.at(i))).left((*files.at(i)).length() - 3) == (*it).text(0)){
					m_repository->removeCatalog(QString::number(i+1));
					it->setText(1,"no");
					break;
				}

		it = it->nextSibling();
	}
};

void RPPGui::slotFinish(){
	if (m_wizard->currentPage() != m_dbmanager){
		QString exename = KStandardDirs::findExe("r++");
		if (exename){
			QStringList arguments;
			arguments<< exename << m_dbname->m_edtname->text();
			if (m_options->m_recursive->isChecked())
				arguments << "-r";
			if (m_options->m_all->isChecked())
				arguments << "-a";
			if (m_options->m_fast->isChecked())
				arguments << "-f";
			for (int i = 0; i < m_docudirectories->m_lbodir->count(); ++i)
				arguments <<( "-d" + m_docudirectories->m_lbodir->text(i));
			for (int i = 0; i < m_directories->m_lbodir->count(); ++i)
				arguments << m_directories->m_lbodir->text(i);
			m_output = new RPPOutput(0, "r++output", WDestructiveClose);
			m_output->show(arguments);
		} else {
			KMessageBox::error(m_wizard, "sorry, can't find r++ executable");
		}

	}
};

void RPPGui::slotManager(const QString& str){
	if (str != "database management")
		return;
	QDir dir(m_datadir);
	dir.setNameFilter("*.db");
	QStringList files = dir.entryList();
	QString entryname;
	for (size_t i = 0; i < files.count(); ++i){
		entryname = (*(files.at(i))).left((*files.at(i)).length() - 3);
		QListViewItem* item = m_dbmanager->m_lstdb->findItem(entryname, 0);
		if ( item != 0)
			if (m_repository->catalogs().find(QString::number(i+1)) != m_repository->catalogs().end())
				item->setText(1, "yes");
			else
				item->setText(1, "no");
		else {
			m_dbmanager->m_lstdb->insertItem(item = new KListViewItem(m_dbmanager->m_lstdb, entryname, "yes"));
			if (m_repository->catalogs().find(QString::number(i+1)) == m_repository->catalogs().end())
				item->setText(1, "no");
		}
	}
};

#include "rppgui.moc"
