#ifndef RPPGUI_H
#define RPPGUI_H

#include <qobject.h>

class CodeInformationRepository;
class CppSupportPart;
/**
	This class is for managing everything related to r++ inside a gui (wizard).
	One can create new databases and delete, enable or disable installed databases.
	@author Jonas B. Jacobi
*/
class RPPGui : public QObject{
	Q_OBJECT
public:
	RPPGui(CodeInformationRepository*, CppSupportPart*);
	~RPPGui();
	/**shows the wizard*/
	void show();

private slots:
	/**close the wizard*/
	void slotClose();
	/**switch whether to show the dbmanager, or the new db creation dialog*/
	void slotNext(bool);
	/**add directories, either to m_directories or to m_docudirectories, depending on m_wizards current page*/
	void slotAddDirectories();
	/**remove directories, either from m_directories or from m_docudirectories, depending on m_wizards current page**/
	void slotRemoveDirectories();
	/**delete all in m_dbmanager selected databases*/
	void slotDeleteDb();
	/**enable all in m_dbmanager selected databases*/
	void slotEnableDb();
	/**disable all in m_dbmanager selected databases*/
	void slotDisableDb();
	/**close wizard(if m_wizard->currentPage() == m_dbmanager) or show the r++ outputwidget*/
	void slotFinish();
	void slotManager(const QString&);

private:
	class KWizard* m_wizard;
	class RPPSelectionWidget* m_selection;
	class RPPDirWidget* m_directories;
	class RPPDbManagerWidget* m_dbmanager;
	class RPPNameWidget* m_dbname;
	class RPPDocumentationWidget* m_docudirectories;
	class RPPOptionsWidget* m_options;
	class RPPOutput* m_output;
	CodeInformationRepository* m_repository;
	QString m_datadir;
	CppSupportPart* m_cpppart;
};

#endif
